package com.yspay.saas;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.yspay.saas.util.CheckSignUtil;
import com.yspay.saas.util.DesUtil;
import com.yspay.saas.util.GenerateSignUtil;
import com.yspay.saas.util.HttpClientUtil;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.util.*;

/**
 * @author shitao
 * @ClassName com.yspay.saas
 * @Description TODO
 * @date 2022/2/10  10:33
 */
public abstract class AbstractGmsTest {
    /***
     * 国密商户私钥
     */
    protected String privateKeyPath = "D:\\data\\test_sm2.sm2";
    /**
     * 国密商户公钥
     */
    protected String publicKeyPath = "D:\\data\\test_sm2.cer";
    /**
     * 国密商户私钥密码
     */
    protected String pwd = "ys123456";
    /**
     * 国密云商服服务公钥（用于响应验签和aesKey加密）
     */
    protected String ysPublicKeyPath = "D:\\data\\sm2businessgate.cer";
    private String localDesKey = "A81F41DE7406CC1566579A751D93CA7A";
    private String proDesKey = "A81F41DE7406CC1566579A751D93CA7A";
    /**
     * 生产环境请求base路径
     */
    public static final String PRO_BASE_URL = "https://ouser.ysepay.com:6443/saas/";
    /**
     * 本地环境
     */
    public static final String LOCAL_BASE_URL = "http://orgmerc.ysepay.com:8880/saas/";
    /*public static final String LOCAL_BASE_URL = "http://localhost:7200/saas/";*/

    /**
     * 发送国密请求
     *
     * @param uri   请求的uri资源名
     * @param param 请求参数；
     * @param cerId 机构号
     * @return
     * @throws Exception
     */
    public String sendPost(String uri, Map<String, String> param, String cerId, EVN evn) throws Exception {
        String paramJsonStr = JSON.toJSONString(param);
        String creData = DesUtil.encryptExtraData(localDesKey, "UTF-8", paramJsonStr);
        Map<String, String> map = new HashMap<String, String>();

        map.put("ver", "1.0");
        map.put("msgCode", "Merchant");
        map.put("method", "addScanMerchant");
        map.put("src", cerId);
        map.put("signType", "SM");
        map.put("charset", "UTF-8");
        map.put("data", creData);
        List<String> keys = new ArrayList<String>(map.keySet());
        Collections.sort(keys);
        StringBuilder sb1 = new StringBuilder();
        for (String k : keys) {
            if ("sign".equals(k) || StringUtils.isEmpty(map.get(k))) {
                continue;
            }
            sb1.append(k).append("=");
            sb1.append(map.get(k));
            sb1.append("&");
        }
        if (sb1.length() > 0) {
            sb1.setLength(sb1.length() - 1);
        }
        String sign = GenerateSignUtil.generateGmSign(map, privateKeyPath, pwd);
        map.put("signValue", sign);
        map.put("reqFlowId", String.valueOf(System.currentTimeMillis()));
        String paramStr = JSONObject.toJSONString(map);
        System.out.println("send 云商服请求参数：" + paramStr);
        String url = "";
        if (evn == EVN.PRO) {
            url = PRO_BASE_URL.concat(uri);
        } else {
            url = LOCAL_BASE_URL.concat(uri);
        }

        String result = HttpClientUtil.sendJson(url, JSONObject.parseObject(paramStr));
        System.out.println("result:" + result);
        //处理响应字符串中base64编码可能添加的换行符
        result = result.replaceAll("\r|\n", "");
        System.out.println("result:" + result);
        Map<String, String> results = JSONObject.parseObject(result, Map.class);
        String signValue=results.get("signValue");
        String data=results.get("data");
        Map<String,String> signParam=JSONObject.parseObject(data,Map.class);
        results.put("signValue",null);
        boolean isFlag = CheckSignUtil.gmsM2CheckContent(signParam, signValue, FileUtils.readFileToByteArray(new File(ysPublicKeyPath)), "UTF-8");
        System.out.println("flag:" + isFlag);
        System.out.println("data:" + results.get("data"));
        String desData = DesUtil.decryptExtraData(localDesKey, "UTF-8", results.get("data"));
        System.out.println(desData);
        return result;

    }

    protected String sendBasicPic(String uri, Map<String, String> param, File file, String fileKey, EVN evn) throws Exception {
        String url = null;
        if (evn == EVN.PRO) {
            url = PRO_BASE_URL.concat(uri);
        } else {
            url = LOCAL_BASE_URL.concat(uri);
        }

        return HttpClientUtil.sendHttpMessage(url, param, file, fileKey);
    }

    /**
     * 测试环境类型
     */
    protected enum EVN {
        PRO, LOCAL
    }
}
